// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

float4x4 tW: WORLD;
float4x4 tV: VIEW;
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//tweakables////

//sobel edge detection
int TexWidth <string uiname="Texture Width";> = 512;
int TexHeight <string uiname="Texture Heihgt";> = 512;

float thresh;

float4x4 tCol ;

//TEXTURES/////////////////////////////

//SOURCE 1 /////////////////////////////////
texture Tex <string uiname="SourceTexture1";>;
float4x4 tTex: TEXTUREMATRIX <string uiname="Source1Transform";>;
sampler Samp = sampler_state
{
    Texture   = (Tex);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    //AddressU  = Clamp;
    //AddressV  = Clamp;
};

//SOURCE 2 /////////////////////////////////
texture Tex2 <string uiname="SourceTexture2";>;
float4x4 tTex2: TEXTUREMATRIX <string uiname="Source2Transform";>;
sampler Samp2 = sampler_state
{
    Texture   = (Tex2);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU  = Clamp;
};

//------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
struct vs2ps
{
    float4 Pos        : POSITION;
    float4 TexCd      : TEXCOORD0;
    float4 TexCd2     : TEXCOORD1;
};


vs2ps VS(
    float4 Pos         : POSITION,
    float4 TexCd       : TEXCOORD0,
    float4 TexCd2      : TEXCOORD1)
{
    vs2ps Out = (vs2ps)0;

    Out.Pos = mul(Pos, tWVP);
    Out.TexCd2 = mul(TexCd, tTex2);
    Out.TexCd = mul(TexCd, tTex);

    return Out;
    }



// -------------------------------------------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------------------------------------------------------------------




//SOBEL//////////////////////////////////////////////////
float4 sobelEdgeDetection(float2 cTex, sampler sTex, float width, float height)
{
	width  = 1 /float (width/2.0);
	height = 1 /float (height/2.0);

	float2 sampleOffsets[8] ={
					-width,	-height, 	// upper row
					 0.0,		-height,
					 width,	-height,
					-width,	 0.0,		// center row
					 width,	 0.0,
					-width,	 height,	// bottom row
					 0.0,		 height,
					 width,	 height,
				};


	int i =0;
	float4 c = .5;
	float2 texCoords;
	float4 texSamples[8];
	float4 vertGradient;
	float4 horzGradient;


	for(i =0; i < 8; i++)
	{
		texCoords = cTex + sampleOffsets[i]; // add sample offsets stored in c10-c17 (inclusive)
		// take sample
		texSamples[i] = tex2D(sTex, texCoords);
		// convert to b&w
		texSamples[i] = dot(texSamples[i], .333333f);
	}

	// VERTICAL Gradient
	vertGradient = -(texSamples[0] + texSamples[5] + 2*texSamples[3]);
	vertGradient += (texSamples[2] + texSamples[7] + 2*texSamples[4]);
	// Horizontal Gradient
	horzGradient = -(texSamples[0] + texSamples[2] + 2*texSamples[1]);
	horzGradient += (texSamples[5] + texSamples[7] + 2*texSamples[6]);

	// we could approximate by adding the abs value..but we have the horse power

	c = sqrt( horzGradient*horzGradient + vertGradient*vertGradient );
	return c;

}

float4 PS_Sobel(float2 cTex: TEXCOORD0): COLOR0
{
   float4 c = sobelEdgeDetection(cTex, Samp, float(TexWidth), float(TexHeight));
   return c;
}

//SIMPLE//////////////////////////////////////////////////
float4 PS(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, In.TexCd);
    float4 col2 = tex2D(Samp2, In.TexCd2);
    float4 col3 = col + col2;
    
    return mul(col3,tCol);

}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Simple {
pass P0
{
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

technique Sobel {
pass P0
{
        VertexShader = compile vs_2_0 VS();
        PixelShader  = compile ps_2_0 PS_Sobel();
    }
}
